'use strict';

var chunkVJKL3UNM_js = require('./chunk-VJKL3UNM.js');
var chunkQEE7GVZV_js = require('./chunk-QEE7GVZV.js');
var chunkH736K5TN_js = require('./chunk-H736K5TN.js');
var env = require('@lsk4/env');
var err = require('@lsk4/err');
var log = require('@lsk4/log');
var common = require('@nestjs/common');
var fishbird = require('fishbird');
var rxjs = require('rxjs');

function _ts_decorate(decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
}
chunkH736K5TN_js.__name(_ts_decorate, "_ts_decorate");
exports.RmqInterceptor = class RmqInterceptor {
  static {
    chunkH736K5TN_js.__name(this, "RmqInterceptor");
  }
  log = log.createLogger(this.constructor.name, {
    ns: "rmq",
    level: "warn"
  });
  async intercept(context, next) {
    const type = context.getType();
    const isLog = chunkQEE7GVZV_js.getRmqConfig("isLog") || false;
    const maxAttempts = chunkQEE7GVZV_js.getRmqConfig("maxAttempts") || 0;
    const errDelay = chunkQEE7GVZV_js.getRmqConfig("errDelay") || 0;
    if (type !== "rmq")
      return next.handle();
    const startedAt = /* @__PURE__ */ new Date();
    const name = context.getHandler().name || context.getClass().name;
    const message = context.getArgByIndex(0);
    const data = message?.data;
    const { fields, properties } = context.getArgByIndex(1);
    const { correlationId } = properties;
    const { redelivered } = fields;
    const messageId = correlationId;
    if (isLog)
      this.log.debug(`[${name}]`, "init", data);
    try {
      if (!data)
        throw new err.Err("!data", {
          status: 400
        });
      const res = await rxjs.lastValueFrom(next.handle());
      const finishedAt = /* @__PURE__ */ new Date();
      const duration = +finishedAt - +startedAt;
      if (isLog)
        this.log.info(`[${name}]`, "success", {
          duration
        });
      const meta = {
        data,
        meta: message.meta,
        startedAt,
        finishedAt,
        duration
      };
      delete chunkQEE7GVZV_js.rmqDeliveryAttempts[messageId];
      return rxjs.of({
        code: 0,
        data: res,
        message: "ok",
        meta
      });
    } catch (err$1) {
      const finishedAt = /* @__PURE__ */ new Date();
      const duration = +finishedAt - +startedAt;
      const attempts = chunkVJKL3UNM_js.inc(chunkQEE7GVZV_js.rmqDeliveryAttempts, messageId);
      const isMaxAttempts = Boolean(redelivered && attempts > maxAttempts);
      const meta = {
        data,
        meta: message.meta,
        startedAt,
        finishedAt,
        duration,
        attempts
      };
      const status = err$1?.status;
      if (status === 200) {
        this.log.warn(`[${name}] skip`, err.Err.getCode(err$1), {
          duration
        });
      } else {
        if (isMaxAttempts) {
          this.log.warn(`[${name}] manyAttempts`, err.Err.getCode(err$1), {
            duration
          });
        } else {
          this.log.error(`[${name}] err`, err.Err.getCode(err$1), {
            duration
          });
        }
        if (env.isDev) {
          this.log.error(`[${name}] err`, err$1);
          this.log.trace(`[${name}] delay`, errDelay);
          await fishbird.delay(errDelay);
        }
      }
      if (isMaxAttempts)
        delete chunkQEE7GVZV_js.rmqDeliveryAttempts[messageId];
      throw new err.Err(err$1, {
        status: isMaxAttempts ? 300 : status || 500,
        meta
      });
    }
  }
};
exports.RmqInterceptor = _ts_decorate([
  common.Injectable()
], exports.RmqInterceptor);
//# sourceMappingURL=out.js.map
//# sourceMappingURL=chunk-ESJKQNR2.js.map