import { inc } from './chunk-VJKL3UNM.js';
import { getRmqConfig, rmqDeliveryAttempts } from './chunk-I7LLFSFU.js';
import { __name } from './chunk-H736K5TN.js';
import { isDev } from '@lsk4/env';
import { Err } from '@lsk4/err';
import { createLogger } from '@lsk4/log';
import { Injectable } from '@nestjs/common';
import { delay } from 'fishbird';
import { lastValueFrom, of } from 'rxjs';

function _ts_decorate(decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
}
__name(_ts_decorate, "_ts_decorate");
var RmqInterceptor = class {
  static {
    __name(this, "RmqInterceptor");
  }
  log = createLogger(this.constructor.name, {
    ns: "rmq",
    level: "warn"
  });
  async intercept(context, next) {
    const type = context.getType();
    const isLog = getRmqConfig("isLog") || false;
    const maxAttempts = getRmqConfig("maxAttempts") || 0;
    const errDelay = getRmqConfig("errDelay") || 0;
    if (type !== "rmq")
      return next.handle();
    const startedAt = /* @__PURE__ */ new Date();
    const name = context.getHandler().name || context.getClass().name;
    const message = context.getArgByIndex(0);
    const data = message?.data;
    const { fields, properties } = context.getArgByIndex(1);
    const { correlationId } = properties;
    const { redelivered } = fields;
    const messageId = correlationId;
    if (isLog)
      this.log.debug(`[${name}]`, "init", data);
    try {
      if (!data)
        throw new Err("!data", {
          status: 400
        });
      const res = await lastValueFrom(next.handle());
      const finishedAt = /* @__PURE__ */ new Date();
      const duration = +finishedAt - +startedAt;
      if (isLog)
        this.log.info(`[${name}]`, "success", {
          duration
        });
      const meta = {
        data,
        meta: message.meta,
        startedAt,
        finishedAt,
        duration
      };
      delete rmqDeliveryAttempts[messageId];
      return of({
        code: 0,
        data: res,
        message: "ok",
        meta
      });
    } catch (err) {
      const finishedAt = /* @__PURE__ */ new Date();
      const duration = +finishedAt - +startedAt;
      const attempts = inc(rmqDeliveryAttempts, messageId);
      const isMaxAttempts = Boolean(redelivered && attempts > maxAttempts);
      const meta = {
        data,
        meta: message.meta,
        startedAt,
        finishedAt,
        duration,
        attempts
      };
      const status = err?.status;
      if (status === 200) {
        this.log.warn(`[${name}] skip`, Err.getCode(err), {
          duration
        });
      } else {
        if (isMaxAttempts) {
          this.log.warn(`[${name}] manyAttempts`, Err.getCode(err), {
            duration
          });
        } else {
          this.log.error(`[${name}] err`, Err.getCode(err), {
            duration
          });
        }
        if (isDev) {
          this.log.error(`[${name}] err`, err);
          this.log.trace(`[${name}] delay`, errDelay);
          await delay(errDelay);
        }
      }
      if (isMaxAttempts)
        delete rmqDeliveryAttempts[messageId];
      throw new Err(err, {
        status: isMaxAttempts ? 300 : status || 500,
        meta
      });
    }
  }
};
RmqInterceptor = _ts_decorate([
  Injectable()
], RmqInterceptor);

export { RmqInterceptor };
//# sourceMappingURL=out.js.map
//# sourceMappingURL=chunk-33KP427H.js.map